/**
 * Copyright (c) 2019 Inomera Research.
 * @author Burak Soykal
 */


#import "FNetmera.h"
#import "FNetmeraService.h"
#import "NetmeraFlutterSdkPlugin.h"

@implementation NetmeraFlutterSdkPlugin

static FlutterPluginRegistrantCallback _registerPlugin = nil;
static NSObject<FlutterPluginRegistrar>* _registrar = nil;

static NSString *const INIT_NETMERA = @"initNetmera";
static NSString *const REQUEST_PUSH_NOTIFICATION_AUTHORIZATION = @"requestPushNotificationAuthorization";
static NSString *const LOGGING = @"logging";
static NSString *const SET_BASE_URL = @"setBaseUrl";
static NSString *const CURRENT_EXTERNAL_ID = @"currentExternalId";
static NSString *const REQUEST_PERMISSIONS_FOR_LOCATION = @"requestPermissionsForLocation";
static NSString *const SET_NETMERA_MAX_ACTIVE_REGIONS = @"setNetmeraMaxActiveRegions";
static NSString *const ENABLE_POPUP_PRESENTATION = @"enablePopupPresentation";
static NSString *const DISABLE_POPUP_PRESENTATION = @"disablePopupPresentation";
static NSString *const ENABLE_PUSH = @"enablePush";
static NSString *const DISABLE_PUSH = @"disablePush";
static NSString *const IS_PUSH_ENABLED = @"isPushEnabled";
static NSString *const TURN_OFF_SENDING_EVENT_AND_USER_UPDATE = @"turnOffSendingEventAndUserUpdate";
static NSString *const FETCH_INBOX = @"fetchInbox";
static NSString *const FETCH_NEXT_PAGE = @"fetchNextPage";
static NSString *const COUNT_FOR_STATUS = @"countForStatus";
static NSString *const INBOX_UPDATE_STATUS = @"inboxUpdateStatus";
static NSString *const FETCH_CATEGORY = @"fetchCategory";
static NSString *const FETCH_NEXT_CATEGORY = @"fetchNextCategory";
static NSString *const SEND_EVENT = @"sendEvent";
static NSString *const UPDATE_USER = @"updateUser";
static NSString *const INIT_SERVICE = @"initService";
static NSString *const SET_API_KEY = @"setApiKey";
static NSString *const HANDLE_PUSH_OBJECT = @"handlePushObject";
static NSString *const HANDLE_LAST_MESSAGE = @"handleLastMessage";
static NSString *const SET_APP_GROUP_NAME = @"setAppGroupName";
static NSString *const HANDLE_INTERACTIVE_ACTION = @"handleInteractiveAction";
static NSString *const UPDATE_STATUS_BY_CATEGORIES = @"updateStatusByCategories";
static NSString *const UPDATE_ALL = @"updateAll";

+ (void)setPluginRegistrantCallback:(FlutterPluginRegistrantCallback)callback {
    _registerPlugin = callback;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"netmera_flutter_sdk"
                                     binaryMessenger:[registrar messenger]];
    NetmeraFlutterSdkPlugin* instance = [[NetmeraFlutterSdkPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    _registrar = registrar;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([INIT_NETMERA isEqualToString:call.method]) {
        NSString* apiKey = call.arguments[@"apiKey"];
        [FNetmera initNetmera: apiKey];
    } else if ([REQUEST_PUSH_NOTIFICATION_AUTHORIZATION isEqualToString:call.method]) {
        [FNetmera requestPushNotificationAuthorization];
    } else if ([SET_BASE_URL isEqualToString:call.method]) {
        [FNetmera setBaseUrl: call];
    } else if ([LOGGING isEqualToString:call.method]) {
        NSNumber* enabled = call.arguments[@"enabled"];
        [FNetmera logging: (BOOL)enabled];
    } else if ([REQUEST_PERMISSIONS_FOR_LOCATION isEqualToString:call.method]) {
        [FNetmera requestPermissionsForLocation];
    } else if ([SET_NETMERA_MAX_ACTIVE_REGIONS isEqualToString:call.method]) {
        [FNetmera setNetmeraMaxActiveRegions: call];
    } else if ([ENABLE_POPUP_PRESENTATION isEqualToString:call.method]) {
        [FNetmera enablePopupPresentation];
    } else if ([DISABLE_POPUP_PRESENTATION isEqualToString:call.method]) {
        [FNetmera disablePopupPresentation];
    }else if ([ENABLE_PUSH isEqualToString:call.method]) {
        [FNetmera enablePush];
    } else if ([DISABLE_PUSH isEqualToString:call.method]) {
        [FNetmera disablePush];
    } else if ([IS_PUSH_ENABLED isEqualToString:call.method]) {
        [FNetmera isPushEnabled: result];
    } else if ([FETCH_INBOX isEqualToString:call.method]) {
        [FNetmera fetchInbox: call result: result];
    } else if ([FETCH_NEXT_PAGE isEqualToString:call.method]) {
        [FNetmera fetchNextPage: result];
    } else if ([CURRENT_EXTERNAL_ID isEqualToString:call.method]) {
        [FNetmera getExternalId: result];
    }else if ([COUNT_FOR_STATUS isEqualToString:call.method]) {
        [FNetmera countForStatus: call result: result];
    } else if ([INBOX_UPDATE_STATUS isEqualToString:call.method]) {
        [FNetmera inboxUpdateStatus: call result: result];
    } else if ([SEND_EVENT isEqualToString:call.method]) {
        [FNetmera sendEvent: call];
    } else if ([UPDATE_USER isEqualToString:call.method]) {
        [FNetmera updateUser: call];
    } else if ([SET_API_KEY isEqualToString:call.method]) {
        [FNetmera setApiKey: call];
    } else if ([HANDLE_PUSH_OBJECT isEqualToString:call.method]) {
        [FNetmera handlePushObject: call result: result];
    } else if ([SET_APP_GROUP_NAME isEqualToString:call.method]) {
        [FNetmera setAppGroupName: call];
    } else if ([TURN_OFF_SENDING_EVENT_AND_USER_UPDATE isEqualToString:call.method]) {
        [FNetmera turnOffSendingEventAndUserUpdate];
    } else if ([INIT_SERVICE isEqualToString:call.method]) {
        [FNetmeraService initService:call registerPlugin:_registerPlugin registrar: _registrar];
    } else if ([FETCH_CATEGORY isEqualToString:call.method]) {
        [FNetmera fetchCategory: call result: result];
    } else if ([FETCH_NEXT_CATEGORY isEqualToString:call.method]) {
        [FNetmera fetchNextCategoryPage: result];
    } else if ([HANDLE_LAST_MESSAGE isEqualToString:call.method]) {
        [FNetmera handleLastMessage: call result: result];
    }else if ([HANDLE_INTERACTIVE_ACTION isEqualToString:call.method]) {
        [FNetmera handleInteractiveAction:call result:result];
    } else if ([UPDATE_STATUS_BY_CATEGORIES isEqualToString:call.method]) {
        [FNetmera updateStatusByCategories:call result:result];
    } else if ([UPDATE_ALL isEqualToString:call.method]) {
        [FNetmera updateAll:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

@end


